import {
  select, // Importing select function from d3 library
  csv, // Importing csv function from d3 library
  scaleLinear, // Importing scaleLinear function from d3 library
  scaleTime, // Importing scaleTime function from d3 library
  extent, // Importing extent function from d3 library
  axisLeft, // Importing axisLeft function from d3 library
  axisBottom, // Importing axisBottom function from d3 library
  format, // Importing format function from d3 library
  line, // Importing line function from d3 library
  area, // Importing area function from d3 library
  curveBasis, // Importing curveBasis function from d3 library
  timeFormat, // Importing timeFormat function from d3 library
} from 'd3'; // Importing necessary functions from d3 library

// Select the SVG element
const svg = select('svg'); // Selecting the SVG element

// Get width and height of the SVG element
const width = +svg.attr('width'); // To get the width of the SVG element
const height = +svg.attr('height'); // To get the height of the SVG element

// Function to format Y-axis tick values
const yAxisTickFormat = (number) => format('.2s')(number); // Format Y-axis tick values

// Function to format X-axis tick values
const xAxisTickFormat = timeFormat('%Y'); // Format X-axis tick values

// Rendering the Area Chart
const render = (data) => {
  // Function to render the area chart with provided data
  // Define accessors for X and Y values
  const xValue = (d) => d.Year; // Define accessor for X value
  const yValue = (d) => d.Expns; // Define accessor for Y value
  // Define labels for X and Y axes
  const xAxisLabel = 'Year'; // Define label for X-axis
  const yAxisLabel = 'Expenses'; // Define label for Y-axis
  // Define margins for the chart
  const margin = {
    top: 10, // Margin from the top
    right: 40, // Margin from the right
    bottom: 88, // Margin from the bottom
    left: 150, // Margin from the left
  };
  // Calculate inner width and height for the chart area
  const innerWidth = width - margin.left - margin.right; // Calculate inner width
  const innerHeight = height - margin.top - margin.bottom; // Calculate inner height
  // Create X scale based on the xValue from the data
  let xScale = scaleTime()
    .domain(extent(data, xValue)) // Define domain for X scale
    .range([0, innerWidth]) // Define range for X scale
    .nice(); // Adjust scale for nice ticks
  // Adjust the xScale domain to start from 2007
  const dataStartDate = new Date(2009, 0, 0); // Define start date for data
  const dataEndDate = extent(data, xValue)[1]; // Get end date from data
  xScale.domain([dataStartDate, new Date(2017, 0, 1)]); // Adjust domain of X scale

  // Create Y scale based on the yValue from the data
  const yScale = scaleLinear()
    .domain(extent(data, yValue)) // Define domain for Y scale
    .range([innerHeight, 0]) // Define range for Y scale
    .nice(); // Adjust scale for nice ticks

  // Append a group element for chart elements
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Create X and Y axis generators
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight) // Define tick size
    .tickPadding(15) // Define tick padding
    .tickFormat(xAxisTickFormat); // Format tick values

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth) // Define tick size
    .tickPadding(10) // Define tick padding
    .tickFormat(yAxisTickFormat); // Format tick values

  // Render Y-axis
  const yAxisG = g.append('g').call(yAxis); // Append Y-axis to SVG
  yAxisG.selectAll('.domain').remove(); // Remove domain line
  yAxisG
    .append('text')
    .attr('class', 'axis-label') // Add class for styling
    .attr('y', -93) // Set Y position
    .attr('x', -innerHeight / 2) // Set X position
    .attr('transform', `rotate(-90)`) // Rotate text
    .attr('text-anchor', 'middle') // Set text anchor
    .text(yAxisLabel); // Set text content

  // Render X-axis
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`); // Translate X-axis

  xAxisG.select('.domain').remove(); // Remove domain line

  xAxisG
    .append('text')
    .attr('class', 'axis-label') // Add class for styling
    .attr('y', 75) // Set Y position
    .attr('x', innerWidth / 2) // Set X position
    .text(xAxisLabel); // Set text content

  // Generate the area under the curve
  const areaGenerator = area()
    .x((d) => xScale(xValue(d)))
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis); // Define curve interpolation

  // Append the area path to the SVG
  g.append('path')
    .attr('class', 'area-chart') // Add class for styling
    .attr('d', areaGenerator(data)); // Set path data
};

// Load data from CSV file
csv(
  'https://gist.githubusercontent.com/nandinimaddula/6b773d56bd45a47ef8367896a94cd4a2/raw/70ed91d14af7612ee2b3bbf6fcd11e316653c4e7/Revenue.csv',
).then((data) => {
  // Convert string data to numeric and date formats
  data.forEach((d) => {
    d.Expns = +d.Expns; // Convert string to number
    d.Year = new Date(d.Year); // Convert string to Date object
  });
  // Render the area chart
  render(data); // Call render function with loaded data
});
